@@ -16,34 +16,139 @@
 namespace steme {
 
 
+/**
+ * Holds likelihoods for every sequence in the text.
+ */
+typedef std::vector< double >              likelihoods_vec_t;        ///< Holds likelihoods for every character in a sequence.
+typedef std::vector< likelihoods_vec_t >   likelihoods_vec_vec_t;    ///< Holds likelihoods for every character in several sequences.
+typedef boost::shared_ptr< likelihoods_vec_vec_t > likelihoods_vec_vec_ptr; ///< Pointer to likelihoods.
+
+
+/// Log likelihood of a W-mer
+inline
+double
+W_mer_log_likelihood( const likelihoods_vec_t & log_likelihoods, size_t offset, size_t W ) {
+    if( log_likelihoods.size() <= offset + W - 1 ) {
+        throw std::invalid_argument( "Offset out of range. " );
+    }
+    return log_likelihoods[ offset + W - 1 ] - ( offset ? log_likelihoods[ offset - 1 ] : 0. );
+}
+
+
+
+
+
+
+/**
+ * Stores minimum likelihoods for each node in a suffix tree up to depth W.
+ */
+struct wmer_likelihoods
+: boost::noncopyable // don't copy large vector around
+{
+    typedef boost::shared_ptr< wmer_likelihoods >  ptr;  ///< Pointer.
+    typedef seqan::String< double >    likelihoods_vec; ///< Type to hold the likelihoods and their bounds.
+
+    likelihoods_vec            min_log;        ///< Minimum log likelihood of all W-mers under this node.
+    double                    LL;             ///< Part of the expected log likelihood under assumption all Z[n] are 0. Used to speed up calculation of LL.
+    size_t                     W;              ///< Width of the W-mers we are evaluating.
+
+    /// Constructor.
+    inline
+    wmer_likelihoods( size_t W )
+    : LL( 0. )
+    , W( W )
+    {
+    }
+
+    /// Return the minimum log likelihood of any W-mer that starts with the prefix represented by this iterator.
+    template< typename TopDownIt >
+    double
+    get_min_log_likelihood( TopDownIt it ) const {
+        return seqan::getProperty( min_log, value( it ) );
+    }
+};
+
+
+
+/**
+ * A background model for a particular motif width, W
+ */
+struct bg_model
+{
+    typedef boost::shared_ptr< bg_model >  ptr;  ///< Pointer.
+
+    wmer_likelihoods               wmer_LLs;       ///< Log likelihoods of the W-mers.
+    const likelihoods_vec_vec_t * base_LLs;       ///< Log likelihoods per-base.
+    zero_order_frequencies         freqs;          ///< The 0-order frequencies.
+
+    bg_model( size_t W, const zero_order_frequencies & freqs, const likelihoods_vec_vec_t * base_LLs = 0 )
+    : wmer_LLs( W )
+    , base_LLs( base_LLs )
+    , freqs( freqs )
+    {
+    }
+
+
+    /// Do we have base-pair resolution log-likelihoods?
+    inline
+    bool
+    have_base_pair_LLs() const {
+        return base_LLs;
+    }
+
+
+    /// Return the minimum log likelihood of any W-mer that starts with the prefix represented by this iterator.
+    template< typename TopDownIt >
+    double
+    get_min_log_likelihood( TopDownIt it ) const {
+        return wmer_LLs.get_min_log_likelihood( it );
+    }
+
+
+    /// Return the log likelihood of the W-mer.
+    inline
+    double
+    wmer_log_likelihood( size_t seq, size_t offset ) const {
+        if( ! have_base_pair_LLs() ) {
+            throw std::logic_error( "Do not have base-pair resolution log likelihoods.");
+        }
+        if( base_LLs->size() <= seq ) {
+            throw std::invalid_argument( "Sequence out of range. " );
+        }
+        return W_mer_log_likelihood( ( *base_LLs )[ seq ], offset, wmer_LLs.W );
+    }
+
+
+    /// Return the log likelihood of the W-mer if we have it otherwise return the log likelihood of the W-mers indexed by the iterator.
+    template< typename TopDownIt >
+    double
+    wmer_log_likelihood( TopDownIt it, size_t seq, size_t offset ) const {
+        // if we have a per-base likelihood use that
+        if( have_base_pair_LLs() ) {
+            return wmer_log_likelihood( seq, offset );
+        } else {
+            // otherwise return the smallest likelihood for that W-mer across entire data
+            return get_min_log_likelihood( it );
+        }
+    }
+};
+
+
 
 
 /**
  * Calculates likelihoods of W-mers in the data given a model. Uses CRTP C++ idiom.
  * The Derived class is the model.
- *
- * \todo Make sure this is not copied around.
  */
 template< typename Derived >
 struct likelihood_calculator
-: boost::noncopyable
 {
+    wmer_likelihoods & likelihoods;
 
-	typedef seqan::String< double >    likelihoods_vec;    ///< Type to hold the likelihoods and their bounds.
-
-	likelihoods_vec            min_log_likelihoods;        ///< Minimum log likelihood of all W-mers under this node.
-	double                     LL;                         ///< Part of the expected log likelihood under assumption all Z[n] are 0. Used to speed up calculation of LL.
-	size_t                     W;                          ///< Width of the W-mers we are evaluating.
-
-
-	/**
-	 * Calculate background evaluations up front.
-	 */
-	likelihood_calculator( size_t W )
-	: LL( 0. )
-	, W( W )
-	{
-	}
+    likelihood_calculator( wmer_likelihoods & likelihoods )
+    : likelihoods( likelihoods )
+    {
+    }
 
 
 	/** Does the calculation. */
@@ -51,10 +156,20 @@ struct likelihood_calculator
 	void
 	calculate( Data & _data ) {
 		using namespace seqan;
-		resizeVertexMap( _data.index, min_log_likelihoods );
-		visit_node( _data, _data.index_root() );
+		resizeVertexMap( _data.index, likelihoods.min_log );
+		visit_node( _data, static_cast< Derived * >( this )->index_root( _data ) );
 	}
 
+
+
+    /** The iterator used for the descent. Can be overridden in derived class. */
+    template< typename Data >
+    typename Data::top_down_it
+    index_root( Data & _data ) {
+        return _data.index_root();
+    }
+
+
 	/**
 	 * Calculates likelihood of W-mers under the model.
 	 *
@@ -62,26 +177,28 @@ struct likelihood_calculator
 	 */
 	template<
 		typename Data,
-		typename TopDownIt
+		typename It
 	>
 	double
-	visit_node( Data & _data, TopDownIt it ) {
+	visit_node( Data & _data, It it ) {
 		using namespace seqan;
 
 		MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
 
+		static_cast< Derived * >( this )->entered_node( it );
+
 		// index into our bounds.
 		typename VertexDescriptor<
-			typename Container< TopDownIt >::Type
+			typename Container< It >::Type
 		>::Type v = value( it );
 
 		double min_ll = 0.; // Return a high bound on the log likelihood if we don't find a lower one.
 
 		// how far down the tree are we?
-		const size_t w = std::min( _data.get_rep_length( it ), W );
+		const size_t w = std::min( _data.get_rep_length( it ), likelihoods.W );
 
 		// did we get to the bottom?
-		if( W == w ) {
+		if( likelihoods.W == w ) {
 
 			// yes, we are at bottom
 			// so calculate the log likelihood
@@ -89,7 +206,7 @@ struct likelihood_calculator
 			boost::tie( W_mer_min_ll, total_ll ) = static_cast< Derived * >( this )->log_likelihoods_for_w_mer( it );
 
 			// update our calculation of part of the LL
-			LL += total_ll;
+			likelihoods.LL += total_ll;
 			min_ll = std::min( min_ll, W_mer_min_ll );
 
 		} else {
@@ -105,78 +222,15 @@ struct likelihood_calculator
 
 		// store the lowest log likelihood
 		MYRRH_ASSERT( min_ll <= 0. ); // perhaps not for empty tree...
-		seqan::assignProperty( min_log_likelihoods, v, min_ll );
+		seqan::assignProperty( likelihoods.min_log, v, min_ll );
+
+        static_cast< Derived * >( this )->left_node( it );
 
 		return min_ll;
 	}
-
-	/// Return the minimum log likelihood of any W-mer that starts with the prefix represented by this iterator.
-	template< typename TopDownIt >
-	double
-	get_min_log_likelihood( TopDownIt it ) const {
-		return seqan::getProperty( min_log_likelihoods, value( it ) );
-	}
 };
 
 
-/**
- * Markov 0-order background model.
- */
-struct Markov0OrderBackgroundModel
-: likelihood_calculator< Markov0OrderBackgroundModel >
-{
-	typedef boost::shared_ptr< Markov0OrderBackgroundModel >  ptr;                ///< A pointer to a Markov0BackgroundModel.
-
-	zero_order_frequencies               freqs;                 ///< The 0-order frequencies.
-
-	/** Constructor. */
-	template< typename Data >
-	Markov0OrderBackgroundModel( Data & _data, const zero_order_frequencies & freqs, size_t W )
-	: likelihood_calculator< Markov0OrderBackgroundModel >( W )
-	, freqs( freqs )
-	{
-		likelihood_calculator< Markov0OrderBackgroundModel >::calculate( _data );
-	}
-
-
-	/** Return the minimum and total log likelihoods of the occurrences of this prefix. */
-	template< typename It >
-	boost::tuple< double, double >
-	log_likelihoods_for_w_mer( It i ) {
-		using namespace seqan;
-
-		typedef typename Container< It >::Type index_t;
-		typedef typename Infix< typename Fibre< index_t, EsaText >::Type const >::Type representative_t;
-		typedef typename Prefix< representative_t >::Type prefix_t;
-		typedef typename Iterator< prefix_t >::Type prefix_it;
-		typedef typename Value< prefix_it >::Type base_t;
-
-		prefix_t p = prefix( representative( i ), W );
-		MS_DEBUG_STRING_VAR( w_mer, p );
-		double ll = 0.;
-		for( prefix_it it = begin( p ); it != end( p ); ++it ) {
-			base_t base = value( it );
-			if( base == unknownValue< base_t >() ) {
-				ll += log_quarter; // treat unknowns as log 1/4
-			} else {
-				ll += freqs.dist_logs[ ordValue( base ) ];
-			}
-		}
-
-		return boost::make_tuple( ll, countOccurrences( i ) );
-	}
-};
-
-
-
-/**
- * Holds likelihoods for every sequence in the text.
- */
-typedef std::vector< double >              likelihoods_vec_t;        ///< Holds likelihoods for every character in a sequence.
-typedef std::vector< likelihoods_vec_t >   likelihoods_vec_vec_t;    ///< Holds likelihoods for every character in several sequences.
-typedef boost::shared_ptr< likelihoods_vec_vec_t > likelihoods_vec_vec_ptr; ///< Pointer to likelihoods.
-
-
 /**
  * Partition a sequence into segments based on whether a predicate is true or not.
  */
@@ -491,50 +545,129 @@ build_model_from_index(
 
 
 
-/// Log likelihood of a W-mer
-inline
-double
-W_mer_log_likelihood( const likelihoods_vec_t & log_likelihoods, size_t offset, size_t W ) {
-    if( log_likelihoods.size() <= offset + W - 1 ) {
-        throw std::invalid_argument( "Offset out of range. " );
+/**
+ * Markov 0-order background model.
+ */
+struct markov_0_order_likelihoods_calculator
+: likelihood_calculator< markov_0_order_likelihoods_calculator >
+{
+    typedef likelihood_calculator< markov_0_order_likelihoods_calculator > base_t;
+    typedef boost::shared_ptr< markov_0_order_likelihoods_calculator >  ptr;                ///< A pointer to a Markov0BackgroundModel.
+
+    zero_order_frequencies         freqs;          ///< The 0-order frequencies.
+
+    /** Constructor. */
+    template< typename Data >
+    markov_0_order_likelihoods_calculator(
+        Data & _data,
+        const zero_order_frequencies & freqs,
+        wmer_likelihoods & likelihoods
+    )
+    : base_t( likelihoods )
+    , freqs( freqs )
+    {
+        base_t::calculate( _data );
     }
-	return log_likelihoods[ offset + W - 1 ] - ( offset ? log_likelihoods[ offset - 1 ] : 0. );
-}
+
+
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    entered_node( It it ) {
+    }
+
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    left_node( It it ) {
+    }
+
+
+    /** Return the minimum and total log likelihoods of the occurrences of this prefix. */
+    template< typename It >
+    boost::tuple< double, double >
+    log_likelihoods_for_w_mer( It i ) {
+        using namespace seqan;
+
+        typedef typename Container< It >::Type index_t;
+        typedef typename Infix< typename Fibre< index_t, EsaText >::Type const >::Type representative_t;
+        typedef typename Prefix< representative_t >::Type prefix_t;
+        typedef typename Iterator< prefix_t >::Type prefix_it;
+        typedef typename Value< prefix_it >::Type base_t;
+
+        prefix_t p = prefix( representative( i ), likelihoods.W );
+        MS_DEBUG_STRING_VAR( w_mer, p );
+        double ll = 0.;
+        for( prefix_it it = begin( p ); it != end( p ); ++it ) {
+            base_t base = value( it );
+            if( base == unknownValue< base_t >() ) {
+                ll += log_quarter; // treat unknowns as log 1/4
+            } else {
+                ll += freqs.dist_logs[ ordValue( base ) ];
+            }
+        }
+
+        return boost::make_tuple( ll, countOccurrences( i ) );
+    }
+};
+
+
+
+
 
 /**
  * A background model created from likelihoods for each base in the sequences.
  */
-struct BackgroundModelFromLikelihoods
-: likelihood_calculator< BackgroundModelFromLikelihoods >
+struct base_to_wmer_likelihoods_calculator
+: likelihood_calculator< base_to_wmer_likelihoods_calculator >
 {
-	typedef boost::shared_ptr< BackgroundModelFromLikelihoods >  ptr;  ///< A pointer to a BackgroundModelFromLikelihoods.
+    typedef likelihood_calculator< base_to_wmer_likelihoods_calculator > base_t;
+	typedef boost::shared_ptr< base_to_wmer_likelihoods_calculator >  ptr;  ///< A pointer to a BackgroundModelFromLikelihoods.
 
-	zero_order_frequencies               freqs;                 ///< The 0-order frequencies.
-	const likelihoods_vec_vec_t &        likelihoods;           ///< The log likelihoods for every base in the sequences.
+	const likelihoods_vec_vec_t &        base_likelihoods;     ///< The log likelihoods for every base in the sequences.
 
 	/** Constructor. */
 	template< typename Data >
-	BackgroundModelFromLikelihoods(
+	base_to_wmer_likelihoods_calculator(
 		Data & _data,
-		const likelihoods_vec_vec_t & log_likelihoods,
-		const zero_order_frequencies & freqs,
-		size_t W
+		const likelihoods_vec_vec_t & base_likelihoods,
+        wmer_likelihoods & likelihoods
 	)
-	: likelihood_calculator< BackgroundModelFromLikelihoods >( W )
-	, freqs( freqs )
-	, likelihoods( log_likelihoods )
+	: base_t( likelihoods )
+	, base_likelihoods( base_likelihoods )
 	{
-		likelihood_calculator< BackgroundModelFromLikelihoods >::calculate( _data );
+	    base_t::calculate( _data );
 	}
 
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    entered_node( It it ) {
+    }
+
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    left_node( It it ) {
+    }
+
+
 	/** Likelihoods for W-mer. */
 	inline
 	double
 	w_mer_log_likelihood( size_t seq, size_t offset ) {
-	    if( likelihoods.size() <= seq ) {
+	    if( base_likelihoods.size() <= seq ) {
 	        throw std::invalid_argument( "Sequence out of range. " );
 	    }
-	    return W_mer_log_likelihood( likelihoods[ seq ], offset, likelihood_calculator< BackgroundModelFromLikelihoods >::W );
+	    return W_mer_log_likelihood( base_likelihoods[ seq ], offset, likelihoods.W );
 	}
 
 
@@ -564,10 +697,100 @@ struct BackgroundModelFromLikelihoods
 };
 
 
+
+
+/**
+ * Calculates the likelihoods of the W-mers using a Markov model.
+ */
+template< typename MarkovModel >
+struct markov_wmer_likelihoods_calculator
+: likelihood_calculator< markov_wmer_likelihoods_calculator< MarkovModel > >
+{
+    typedef likelihood_calculator< markov_wmer_likelihoods_calculator< MarkovModel > > base_t;
+    typedef MarkovModel markov_model_t;
+
+    const markov_model_t &    model;
+    std::vector< double >     current_evals;
+
+    template< typename Data >
+    markov_wmer_likelihoods_calculator(
+        Data & _data,
+        const MarkovModel & model,
+        wmer_likelihoods & likelihoods
+    )
+    : base_t( likelihoods )
+    , model( model )
+    {
+        base_t::calculate( _data );
+    }
+
+
+    /** The iterator used for the descent. */
+    template< typename Data >
+    typename Data::top_down_history_it
+    index_root( Data & _data ) {
+        return typename Data::top_down_history_it( _data.index );
+    }
+
+
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    entered_node( It it ) {
+        using namespace seqan;
+        MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
+
+        // work out where we are
+        const size_t length = std::min( base_t::likelihoods.W, repLength( it ) );
+        const size_t offset = repLength( it ) - parentEdgeLength( it );
+
+        // resize evaluations
+        MYRRH_ASSERT( current_evals.size() >= offset );
+        current_evals.resize( offset );
+
+        // evaluate part of the representative
+        typedef typename Container< It >::Type IndexT;
+        typedef typename Infix< typename Fibre< IndexT, EsaText >::Type const>::Type ReprT;
+        typedef typename boost::range_iterator< ReprT >::type range_it;
+        range_it begin = boost::begin( representative( it ) );
+        model.evaluate_in_context(
+            begin,
+            begin + offset,
+            begin + length,
+            boost::make_function_output_iterator( make_push_back_cumulative( current_evals ) )
+        );
+    }
+
+    /**
+     * Called by base class
+     */
+    template< typename It >
+    void
+    left_node( It it ) {
+    }
+
+    /**
+     * Return the minimum and total log likelihoods of the occurrences of this prefix.
+     */
+    template< typename It >
+    boost::tuple< double, double >
+    log_likelihoods_for_w_mer( It it ) {
+        using namespace seqan;
+        MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
+        MYRRH_ASSERT( W == current_evals.size() );
+        const double it_LL = current_evals.back();
+        return boost::make_tuple( it_LL, countOccurrences( it ) * it_LL );
+    }
+};
+
+
+
 /// Meta-function to choose type.
 template< typename Spec = default_spec >
 struct background_model_meta {
-	typedef BackgroundModelFromLikelihoods type; ///< Background model type.
+    typedef bg_model type; ///< Background model type.
 };
 
 
