@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright John Reid 2011
+# Copyright John Reid 2011, 2012
 #
 
 """
@@ -10,7 +10,11 @@ Models for STEME web app.
 
 import uuid
 from datetime import datetime
-from flaskext.sqlalchemy import SQLAlchemy
+
+try:
+    from flask.ext.sqlalchemy import SQLAlchemy
+except NameError:
+    from flaskext.sqlalchemy import SQLAlchemy
 
 from .application import app
 db = SQLAlchemy(app)
