@@ -22,6 +22,7 @@ def version_string():
 from ._stempy_build import *
 from ._stempy_build import _dummy_fn, _debug, _using_old_bg_model, _python_debug_build
 from ._meme_like_output import MemeLikeOutput
+from ._minimal_meme_output import MinimalMemeOutput
     
 from cookbook.named_tuple import namedtuple
 from cookbook.timer import SimpleTimer, Timer
@@ -408,6 +409,19 @@ def parse_options(additional_options_adder=None, option_parser=None):
     return options, args
 
 
+def get_markov_model_create_fn(options):
+    """Return a function that will create a Markov model of the order specified in the options.
+    """
+    return globals()['create_markov_model_order_from_index_%d' % options.bg_model_order]
+    
+
+def get_bg_model_from_markov_model_fn(options):
+    """Return a function that will create a background model from a Markov model of the order specified in the options.
+    """
+    return globals()['create_bg_model_from_Markov_model_%d' % options.bg_model_order]
+
+
+
 def complement(b):
     if 'A' == b: return 'T'
     if 'C' == b: return 'G'
@@ -1984,6 +1998,7 @@ class Algorithm(object):
             #
             # Initialise outputs
             #
+            self._initialise_minimal_meme_output()
             self._initialise_meme_like_output()
             self._initialise_html_output()
             
@@ -2000,6 +2015,14 @@ class Algorithm(object):
             output = MemeLikeOutput(open(self.meme_like_output_file, 'w'))
             output.initialise(self)
             self.outputs.append(output)
+
+            
+    def _initialise_minimal_meme_output(self):
+        "Initialise minimal MEME output"
+        self.minimal_meme_output_file = os.path.join(self.options.output_dir, 'steme.txt')
+        output = MinimalMemeOutput(open(self.minimal_meme_output_file, 'w'))
+        output.initialise(self)
+        self.outputs.append(output)
     
 
     def _initialise_html_output(self):
